import { Pipe, PipeTransform, NgZone } from "@angular/core";
import { Observable } from 'rxjs/Observable';
import { Observer } from 'rxjs/Observer';
import { AppSettings } from '../app-settings';
import { TranslateService } from 'ng2-translate';
import * as moment from 'moment';

interface processOutput {
  text: string; // Convert timestamp to string
  timeToUpdate: number; // Time until update in milliseconds
}

@Pipe({
  name: 'timeAgo',
  pure: true
})
export class TimeAgoPipe implements PipeTransform {
  static DATE_FORMAT = 'DD/MM/YY HH:mm:ss';
  constructor(public translate: TranslateService, private ngZone: NgZone) { }

  private process = (timestamp: number, rootTranslateKey: string): processOutput => {
    let text: string;
    let timeToUpdate: number;

    const now = new Date();

    // Time ago in milliseconds
    const timeAgo: number = now.getTime() - timestamp;

    const seconds = timeAgo / 1000;
    const minutes = seconds / 60;
    const hours = minutes / 60;
    const days = hours / 24;
    const months = days / 30.416;
    const years = days / 365;

    if (timestamp == 0) {
      text = this.translate.instant(rootTranslateKey + '.NEVER_CONNECTED');
    } else if (seconds <= 10) {
      text = this.translate.instant(rootTranslateKey + '.MSG_00');
    } else if (seconds <= 45) {
      text = this.translate.instant(rootTranslateKey + '.MSG_01');
    } else if (seconds <= 90) {
      text = this.translate.instant(rootTranslateKey + '.MSG_02');
    } else if (minutes <= 50) {
      text = this.translate.instant(rootTranslateKey + '.MSG_03', { value: Math.round(minutes)});
    } else if (hours <= 1.5) {
      text = this.translate.instant(rootTranslateKey + '.MSG_04');
    } else if (hours <= 22) {
      text = this.translate.instant(rootTranslateKey + '.MSG_05', { value: Math.round(hours)});
    } else if (hours <= 36) {
      text = this.translate.instant(rootTranslateKey + '.MSG_06');
    } else if (days <= 25) {
      text = this.translate.instant(rootTranslateKey + '.MSG_07', { value: Math.round(days)});
    } else if (months <= 1.5) {
      text = this.translate.instant(rootTranslateKey + '.MSG_08');
    } else if (months <= 11.5) {
      text = this.translate.instant(rootTranslateKey + '.MSG_09', { value: Math.round(months)});
    } else if (years <= 1.5) {
      text = this.translate.instant(rootTranslateKey + '.MSG_10');
    } else {
      text = this.translate.instant(rootTranslateKey + '.MSG_11', { value: Math.round(years)});
    }

    if (minutes < 1) {
      // update every 2 secs
      timeToUpdate = 2 * 1000;
    } else if (hours < 1) {
      // update every 30 secs
      timeToUpdate = 30 * 1000;
    } else if (days < 1) {
      // update every 5 mins
      timeToUpdate = 300 * 1000;
    } else {
      // update every hour
      timeToUpdate = 3600 * 1000;
    }

    return {
      text,
      timeToUpdate
    };
  }

  public transform = (value: Date, rootTranslateKey: string): Observable<string> => {
    // time value in milliseconds
    let timestamp = value.getTime();
    if (value.getFullYear() == 1970) {
      timestamp = 0;
    }

    let timeoutID: any;

    return Observable.create((observer: Observer<string>) => {
      let latestText = '';

      // Repeatedly set new timeouts for new update checks.
      const registerUpdate = () => {
        const processOutput = this.process(timestamp, rootTranslateKey);
        if (processOutput.text !== latestText) {
          latestText = processOutput.text;
          this.ngZone.run(() => {
            observer.next(latestText);
          });
        }
        timeoutID = setTimeout(registerUpdate, processOutput.timeToUpdate);
      };

      this.ngZone.runOutsideAngular(registerUpdate);

      // Return teardown function
      const teardownFunction = () => {
        if (timeoutID) {
          clearTimeout(timeoutID);
        }
      };
      return teardownFunction;
    });
  }
}