import { Injectable } from '@angular/core';
import { Http, XHRBackend, RequestOptions, Response, Headers, Request, RequestMethod } from "@angular/http";
import { Observable } from "rxjs";
import { AppSettings } from "../app-settings";
import { RequestOptionsUtil } from '../util/request.options.util';
import { MessageConstants } from "../util/messages.constants";
import { ErrorObservable } from "rxjs/observable/ErrorObservable";
import { WpsLocalStorage } from '../local-storage'
import { Login } from '../util/login';

@Injectable()
export class HttpClientService extends Http {

    constructor(backend: XHRBackend, defaultOptions: RequestOptions, private loginUtil: Login) {
        super(backend, defaultOptions);
    }

    put(url: string, body: any, options?: RequestOptions): Observable<Response> {
        return this.doRequest(RequestMethod.Put, url, options, body);
    }

    post(url: string, body: any, options?: RequestOptions): Observable<Response> {
        return this.doRequest(RequestMethod.Post, url, options, body);
    }

    get(url: string, options?: RequestOptions): Observable<Response> {
        return this.doRequest(RequestMethod.Get, url, options);
    }

    delete(url: string, options?: RequestOptions): Observable<Response> {
        return this.doRequest(RequestMethod.Delete, url, options);
    }

    /**
    * Do the request and if the token expires, the method refreshs the token.
    * 
    * @param method The HTTP method.
    * @param url The URL to do the request.
    * @param options The request options with headers.
    * @param body The JSON to put in the body request.
    */
    private doRequest(method: RequestMethod, url: string, options?: RequestOptions, body?: any): Observable<Response> {
        let request: Request = this.getRequest(method, url, options, body);
        return super.request(request).catch((error: Response) => {
            if (error.status === 401) {
                console.log('Refreshing token ...');
                return this.doRefreshToken().flatMap(
                    (response: Response) => {
                        console.log('Refreshing token ... OK!');
                        this.updateTokenInfo(response.json());
                        return this.doRequest(method, url, RequestOptionsUtil.setAccessToken(options), body);
                    }
                ).catch((error: Response) => {
                    console.log('Refreshing token ... Error!');
                    console.log(error);
                    return this.throwRefreshToken();
                })
            } else {
                return Observable.throw(error);
            }
        });
    }

    /**
     * Get the request to do.
     * 
     * @param method The HTTP method.
     * @param url The URL to do the request.
     * @param options The request options with headers.
     * @param body The JSON to put in the body request.
     */
    private getRequest(method: RequestMethod, url: string, options?: RequestOptions, body?: any): Request {
        options.body = body;
        let request: Request = new Request(options);
        request.method = method;
        request.url = url;
        return request;
    }

    /**
     * Do the call to refresh the token.
     */
    private doRefreshToken(): Observable<Response> {
        let header = { 'Content-Type': 'application/json', 'Authorization': 'Basic ' + AppSettings.AUTH };
        let headers = new Headers(header);
        let options = new RequestOptions({ headers: headers });
        let refreshToken = WpsLocalStorage.getItem('refresh_token');
        return super.post(AppSettings.endPointRefreshToken(refreshToken), JSON.stringify(''), options);
    }

    /**
     * Throw the error of refreshing the token and redirect to login.
     */
    private throwRefreshToken(): ErrorObservable {
        this.loginUtil.tokenCheck(MessageConstants.SESSION_EXPIRED);
        WpsLocalStorage.setItem('refresh_token_expired', 'true');
        return Observable.throw({});
    }

    /**
     * Update tje token information in the local storage.
     * 
     * @param response The JSON OAuth 2.0 response.
     */
    private updateTokenInfo(response: any): void {
        console.dir(response);
        WpsLocalStorage.setItem('token', response.access_token);
        WpsLocalStorage.setItem('access_token', response.access_token);
        WpsLocalStorage.setItem('token_type', response.token_type);
        WpsLocalStorage.setItem('refresh_token', response.refresh_token);
        WpsLocalStorage.setItem('expires_in', response.expires_in);
    }
}